@@ -17,9 +17,11 @@
 
 #' @include arrow-package.R
 
-#' @title Array class
-#' @description Array base type. Immutable data array with some logical type
-#' and some length.
+#' @title Arrow Arrays
+#' @description An `Array` is an immutable data array with some logical type
+#' and some length. Most logical types are contained in the base
+#' `Array` class; there are also subclasses for `DictionaryArray`, `ListArray`,
+#' and `StructArray`.
 #' @usage NULL
 #' @format NULL
 #' @docType class
@@ -30,6 +32,15 @@
 #' * `x`: an R vector, list, or `data.frame`
 #' * `type`: an optional [data type][data-type] for `x`. If omitted, the type
 #'    will be inferred from the data.
+#'
+#' `Array$create()` will return the appropriate subclass of `Array`, such as
+#' `DictionaryArray` when given an R factor.
+#'
+#' To compose a `DictionaryArray` directly, call `DictionaryArray$create()`,
+#' which takes two arguments:
+#' * `x`: an R vector or `Array` of integers for the dictionary indices
+#' * `dict`: an R vector or `Array` of dictionary values (like R factor levels
+#'   but not limited to strings only)
 #' @section Usage:
 #'
 #' ```
@@ -54,8 +65,16 @@
 #' - `$data()`: return the underlying [ArrayData][ArrayData]
 #' - `$as_vector()`: convert to an R vector
 #' - `$ToString()`: string representation of the array
-#' - `$Slice(offset, length = NULL)` : Construct a zero-copy slice of the array with the indicated offset and length. If length is `NULL`, the slice goes until the end of the array.
+#' - `$Slice(offset, length = NULL)`: Construct a zero-copy slice of the array
+#'    with the indicated offset and length. If length is `NULL`, the slice goes
+#'    until the end of the array.
+#' - `$Take(i)`: return an `Array` with values at positions given by integers
+#'    (R vector or Array Array) `i`.
+#' - `$Filter(i)`: return an `Array` with values at positions where logical
+#'    vector (or Arrow boolean Array) `i` is `TRUE`.
 #' - `$RangeEquals(other, start_idx, end_idx, other_start_idx)` :
+#' - `$cast(target_type, safe = TRUE, options = cast_options(safe))`: Alter the
+#'    data in the array to change its type.
 #' - `$View(type)`: Construct a zero-copy view of this array with the given type.
 #' - `$Validate()` : Perform any validation checks to determine obvious inconsistencies
 #'    within the array's internal data. This can be an expensive check, potentially `O(length)`
@@ -85,6 +104,26 @@ Array <- R6Class("Array",
         shared_ptr(Array, Array__Slice2(self, offset, length))
       }
     },
+    Take = function(i) {
+      if (is.numeric(i)) {
+        i <- as.integer(i)
+      }
+      if (is.integer(i)) {
+        i <- Array$create(i)
+      }
+      if (inherits(i, "ChunkedArray")) {
+        return(shared_ptr(ChunkedArray, Array__TakeChunked(self, i)))
+      }
+      assert_is(i, "Array")
+      shared_ptr(Array, Array__Take(self, i))
+    },
+    Filter = function(i) {
+      if (is.logical(i)) {
+        i <- Array$create(i)
+      }
+      assert_is(i, "Array")
+      shared_ptr(Array, Array__Filter(self, i))
+    },
     RangeEquals = function(other, start_idx, end_idx, other_start_idx) {
       assert_is(other, "Array")
       Array__RangeEquals(self, other, start_idx, end_idx, other_start_idx)
@@ -107,35 +146,6 @@ Array <- R6Class("Array",
     type = function() DataType$create(Array__type(self))
   )
 )
-
-DictionaryArray <- R6Class("DictionaryArray", inherit = Array,
-  public = list(
-    indices = function() Array$create(DictionaryArray__indices(self)),
-    dictionary = function() Array$create(DictionaryArray__dictionary(self))
-  )
-)
-
-StructArray <- R6Class("StructArray", inherit = Array,
-  public = list(
-    field = function(i) Array$create(StructArray__field(self, i)),
-    GetFieldByName = function(name) Array$create(StructArray__GetFieldByName(self, name)),
-    Flatten = function() map(StructArray__Flatten(self), ~ Array$create(.x))
-  )
-)
-
-ListArray <- R6Class("ListArray", inherit = Array,
-  public = list(
-    values = function() Array$create(ListArray__values(self)),
-    value_length = function(i) ListArray__value_length(self, i),
-    value_offset = function(i) ListArray__value_offset(self, i),
-    raw_value_offsets = function() ListArray__raw_value_offsets(self)
-  ),
-  active = list(
-    value_type = function() DataType$create(ListArray__value_type(self))
-  )
-)
-
-# Add a class method
 Array$create <- function(x, type = NULL) {
   if (!inherits(x, "externalptr")) {
     x <- Array__from_vector(x, type)
@@ -151,8 +161,125 @@ Array$create <- function(x, type = NULL) {
   a
 }
 
+#' @rdname array
+#' @usage NULL
+#' @format NULL
+#' @export
+DictionaryArray <- R6Class("DictionaryArray", inherit = Array,
+  public = list(
+    indices = function() Array$create(DictionaryArray__indices(self)),
+    dictionary = function() Array$create(DictionaryArray__dictionary(self))
+  )
+)
+DictionaryArray$create <- function(x, dict = NULL) {
+  if (is.factor(x)) {
+    # The simple case: converting a factor.
+    # Ignoring `dict`; should probably error if dict is not NULL
+    return(Array$create(x))
+  }
+
+  assert_that(!is.null(dict))
+  if (!is.Array(x)) {
+    x <- Array$create(x)
+  }
+  if (!is.Array(dict)) {
+    dict <- Array$create(dict)
+  }
+  type <- DictionaryType$create(x$type, dict$type)
+  shared_ptr(DictionaryArray, DictionaryArray__FromArrays(type, x, dict))
+}
+
+#' @rdname array
+#' @usage NULL
+#' @format NULL
+#' @export
+StructArray <- R6Class("StructArray", inherit = Array,
+  public = list(
+    field = function(i) Array$create(StructArray__field(self, i)),
+    GetFieldByName = function(name) Array$create(StructArray__GetFieldByName(self, name)),
+    Flatten = function() map(StructArray__Flatten(self), ~ Array$create(.x))
+  )
+)
+
+#' @rdname array
+#' @usage NULL
+#' @format NULL
+#' @export
+ListArray <- R6Class("ListArray", inherit = Array,
+  public = list(
+    values = function() Array$create(ListArray__values(self)),
+    value_length = function(i) ListArray__value_length(self, i),
+    value_offset = function(i) ListArray__value_offset(self, i),
+    raw_value_offsets = function() ListArray__raw_value_offsets(self)
+  ),
+  active = list(
+    value_type = function() DataType$create(ListArray__value_type(self))
+  )
+)
+
 #' @export
 length.Array <- function(x) x$length()
 
 #' @export
 as.vector.Array <- function(x, mode) x$as_vector()
+
+filter_rows <- function(x, i, ...) {
+  # General purpose function for [ row subsetting with R semantics
+  # Based on the input for `i`, calls x$Filter, x$Slice, or x$Take
+  nrows <- x$num_rows %||% x$length() # Depends on whether Array or Table-like
+  if (inherits(i, "array_expression")) {
+    # Evaluate it
+    i <- as.vector(i)
+  }
+  if (is.logical(i)) {
+    if (isTRUE(i)) {
+      # Shortcut without doing any work
+      x
+    } else {
+      i <- rep_len(i, nrows) # For R recycling behavior; consider vctrs::vec_recycle()
+      x$Filter(i)
+    }
+  } else if (is.numeric(i)) {
+    if (all(i < 0)) {
+      # in R, negative i means "everything but i"
+      i <- setdiff(seq_len(nrows), -1 * i)
+    }
+    if (is.sliceable(i)) {
+      x$Slice(i[1] - 1, length(i))
+    } else if (all(i > 0)) {
+      x$Take(i - 1)
+    } else {
+      stop("Cannot mix positive and negative indices", call. = FALSE)
+    }
+  } else if (is.Array(i, INTEGER_TYPES)) {
+    # NOTE: this doesn't do the - 1 offset
+    x$Take(i)
+  } else if (is.Array(i, "bool")) {
+    x$Filter(i)
+  } else {
+    # Unsupported cases
+    if (is.Array(i)) {
+      stop("Cannot extract rows with an Array of type ", i$type$ToString(), call. = FALSE)
+    }
+    stop("Cannot extract rows with an object of class ", class(i), call.=FALSE)
+  }
+}
+
+#' @export
+`[.Array` <- filter_rows
+
+is.sliceable <- function(i) {
+  # Determine whether `i` can be expressed as a $Slice() command
+  is.numeric(i) &&
+    length(i) > 0 &&
+    all(i > 0) &&
+    identical(as.integer(i), i[1]:i[length(i)])
+}
+
+is.Array <- function(x, type = NULL) {
+  is_it <- inherits(x, c("Array", "ChunkedArray"))
+  if (is_it && !is.null(type)) {
+    is_it <- x$type$ToString() %in% type
+  }
+  is_it
+}
