@@ -15,6 +15,7 @@
 # specific language governing permissions and limitations
 # under the License.
 
+#' @include record-batch.R
 #' @title Table class
 #' @description A Table is a sequence of [chunked arrays][ChunkedArray]. They
 #' have a similar interface to [record batches][RecordBatch], but they can be
@@ -65,6 +66,11 @@
 #' - `$Slice(offset, length = NULL)`: Create a zero-copy view starting at the
 #'    indicated integer offset and going for the given length, or to the end
 #'    of the table if `NULL`, the default.
+#' - `$Take(i)`: return an `Table` with rows at positions given by
+#'    integers `i`. If `i` is an Arrow `Array` or `ChunkedArray`, it will be
+#'    coerced to an R vector before taking.
+#' - `$Filter(i)`: return an `Table` with rows at positions where logical
+#'    vector or Arrow boolean-type `(Chunked)Array` `i` is `TRUE`.
 #' - `$serialize(output_stream, ...)`: Write the table to the given
 #'    [OutputStream]
 #' - `$cast(target_schema, safe = TRUE, options = cast_options(safe))`: Alter
@@ -132,6 +138,29 @@ Table <- R6Class("Table", inherit = Object,
         shared_ptr(Table, Table__Slice2(self, offset, length))
       }
     },
+    Take = function(i) {
+      if (is.numeric(i)) {
+        i <- as.integer(i)
+      }
+      if (is.integer(i)) {
+        i <- Array$create(i)
+      }
+      if (inherits(i, "ChunkedArray")) {
+        return(shared_ptr(Table, Table__TakeChunked(self, i)))
+      }
+      assert_is(i, "Array")
+      shared_ptr(Table, Table__Take(self, i))
+    },
+    Filter = function(i) {
+      if (is.logical(i)) {
+        i <- Array$create(i)
+      }
+      if (inherits(i, "ChunkedArray")) {
+        return(shared_ptr(Table, Table__FilterChunked(self, i)))
+      }
+      assert_is(i, "Array")
+      shared_ptr(Table, Table__Filter(self, i))
+    },
 
     Equals = function(other) {
       Table__Equals(self, other)
