@@ -17,7 +17,31 @@
 
 context("Compressed.*Stream")
 
+test_that("codec_is_available", {
+  expect_true(codec_is_available("uncompressed")) # Always true
+  expect_error(codec_is_available("sdfasdf"), "'arg' should be one of")
+  skip_if_not_available("gzip")
+  expect_true(codec_is_available("gzip"))
+  expect_true(codec_is_available("GZIP"))
+})
+
+if (identical(Sys.getenv("APPVEYOR"), "True")) {
+  test_that("Compression codecs are included in the Windows build", {
+    expect_true(codec_is_available("lz4"))
+    expect_true(codec_is_available("zstd"))
+  })
+}
+
+test_that("Codec attributes", {
+  skip_if_not_available("gzip")
+  cod <- Codec$create("gzip")
+  expect_equal(cod$name, "gzip")
+  # TODO: implement $level
+  expect_error(cod$level)
+})
+
 test_that("can write Buffer to CompressedOutputStream and read back in CompressedInputStream", {
+  skip_if_not_available("gzip")
   buf <- buffer(as.raw(sample(0:255, size = 1024, replace = TRUE)))
 
   tf1 <- tempfile()
